@@ -63,6 +63,12 @@ class Testpacodemo(BaseTestModelLoader):
         ecrpipe = self.project['netenv']['pacodemo']['demo']['us-west-2']['applications']['app'].groups['cicd'].resources['ecrpipe']
         assert ecrpipe.source['ecr'].type, 'ECR.Source'
 
+        # Test CodeBuild
+        codebuild = deploy_pipeline.stages['build']['codebuild']
+        ecs = codebuild.release_phase.ecs
+        assert schemas.IDeploymentPipelineBuildReleasePhase(codebuild.release_phase)
+        assert ecs[0].command, 'docker rake:deploy'
+
     def test_ne_vpc(self):
         vpc = self.project['netenv']['pacodemo']['demo']['us-west-2'].network.vpc
         assert vpc.enable_dns_support == True
@@ -366,6 +372,7 @@ class Testpacodemo(BaseTestModelLoader):
         demo_env = self.project['netenv']['pacodemo']['demo']['us-west-2']
         cluster = demo_env['applications']['app'].groups['container'].resources['ecs_cluster']
         assert schemas.IECSCluster.providedBy(cluster)
+        assert cluster.capacity_providers[0].base, 1
         ecs_service_config = demo_env['applications']['app'].groups['container'].resources['ecs_services']
 
         assert ecs_service_config.setting_groups['app_pool'].secrets[0].name, 'TWO_SECRET'
@@ -387,6 +394,10 @@ class Testpacodemo(BaseTestModelLoader):
         assert simple_app_service.target_tracking_scaling_policies['cpu'].predefined_metric, 'ECSServiceAverageCPUUtilization'
         assert simple_app_service.target_tracking_scaling_policies['cpu'].target, 70
 
+        # Capacity Providers
+        assert simple_app_service.capacity_providers[0].base, 1
+        assert simple_app_service.capacity_providers[0].weight, 1
+
     def test_cognito(self):
         auth_app = self.demo_app.groups['auth']
         userpool = auth_app.resources['userpool']
